#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Custom widgets in a notebook
# 
# The notebook explore a couple of ways to interact with the user and modifies the output based on these interactions. This is inspired from the examples from [ipwidgets](http://ipywidgets.readthedocs.io/).

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## List of widgets
# 
# [Widget List](https://ipywidgets.readthedocs.io/en/latest/examples/Widget%20List.html)

# In[2]:


import ipywidgets
import datetime


# In[3]:


obj = ipywidgets.DatePicker(
    description='Pick a Date',
    disabled=False,
    value=datetime.datetime.now(),
)
obj


# In[4]:


obj.value


# ## Events

# In[5]:


from IPython.display import display
button = ipywidgets.Button(description="Click Me!")
display(button)

def on_button_clicked(b):
    print("Button clicked.")

button.on_click(on_button_clicked)


# In[6]:


int_range = ipywidgets.IntSlider()
display(int_range)

def on_value_change(change):
    print(change['new'])

int_range.observe(on_value_change, names='value')


# ## matplotlib

# In[7]:


get_ipython().run_line_magic('matplotlib', 'inline')
import matplotlib.pyplot as plt


# In[8]:


import networkx as nx

def random_lobster(n, m, k, p):
    return nx.random_lobster(n, p, p / m)

def powerlaw_cluster(n, m, k, p):
    return nx.powerlaw_cluster_graph(n, m, p)

def erdos_renyi(n, m, k, p):
    return nx.erdos_renyi_graph(n, p)

def newman_watts_strogatz(n, m, k, p):
    return nx.newman_watts_strogatz_graph(n, k, p)

def plot_random_graph(n, m, k, p, generator):
    g = generator(n, m, k, p)
    nx.draw(g)
    plt.show()


# In[9]:


ipywidgets.interact(plot_random_graph, n=(2,30), m=(1,10), k=(1,10), p=(0.0, 1.0, 0.001),
         generator={
             'lobster': random_lobster,
             'power law': powerlaw_cluster,
             'Newman-Watts-Strogatz': newman_watts_strogatz,
             'Erdős-Rényi': erdos_renyi,
         });


# ## Custom widget - text
# 
# [Building a Custom Widget - Hello World](http://ipywidgets.readthedocs.io/en/latest/examples/Widget%20Custom.html).

# In[10]:


import ipywidgets as widgets
from traitlets import Unicode, validate

class HelloWidget(widgets.DOMWidget):
    _view_name = Unicode('HelloView').tag(sync=True)
    _view_module = Unicode('hello').tag(sync=True)
    _view_module_version = Unicode('0.1.0').tag(sync=True)
    value = Unicode('Hello World! - ').tag(sync=True)


# In[11]:


get_ipython().run_cell_magic('javascript', '', 'require.undef(\'hello\');\n\ndefine(\'hello\', ["@jupyter-widgets/base"], function(widgets) {\n\n    var HelloView = widgets.DOMWidgetView.extend({\n\n        render: function() {\n            this.value_changed();\n            this.model.on(\'change:value\', this.value_changed, this);\n        },\n\n        value_changed: function() {\n            this.el.textContent = this.model.get(\'value\');\n        },\n    });\n\n    return {\n        HelloView : HelloView\n    };\n});\n')


# In[12]:


w = HelloWidget()
w


# In[13]:


w.value = 'changed the value'


# ## Custom widget - html - svg - events
# 
# See [Low Level Widget Tutorial](http://ipywidgets.readthedocs.io/en/latest/examples/Widget%20Low%20Level.html), [CircleView](https://github.com/paul-shannon/notebooks/blob/master/study/CircleView.ipynb). The following example links a custom widget and a sliding bar which defines the radius of circle to draw. See [Linking two similar widgets](http://ipywidgets.readthedocs.io/en/latest/examples/Widget%20Basics.html#Linking-two-similar-widgets). The information (circles, radius) is declared in a python class *CircleWidget* and available in the javascript code in two places: the widget (``this.model``) and the view itself (used to connect event to it). Finally, a link is added between two values: value from the first widget (sliding bar) and radius from the second widget (*CircleWidget*).

# In[14]:


get_ipython().run_cell_magic('javascript', '', "require.config({\n    paths: {\n        d3: '//cdnjs.cloudflare.com/ajax/libs/d3/3.4.8/d3.min'\n    },\n});\n")


# In[15]:


import ipywidgets
from traitlets import Int, Unicode, Tuple, CInt, Dict, validate

class CircleWidget(ipywidgets.DOMWidget):
    _view_name = Unicode('CircleView').tag(sync=True)
    _view_module = Unicode('circle').tag(sync=True)
    radius = Int(100).tag(sync=True)
    circles = Tuple().tag(sync=True)
    width = Int().tag(sync=True)
    height = Int().tag(sync=True)
    radius = Int().tag(sync=True)
    def __init__(self, **kwargs):
        super(ipywidgets.DOMWidget, self).__init__(**kwargs)
        self.width = kwargs.get('width', 500)
        self.height = kwargs.get('height', 100)
        self.radius = 1
    def drawCircle(self, x, y, fillColor="white", borderColor="black"):
        newCircle = {"x": x,  "y": y, "radius": self.radius * 10, "fillColor": fillColor, "borderColor": borderColor}
        self.circles = self.circles + (newCircle,)


# In[16]:


get_ipython().run_cell_magic('javascript', '', '"use strict";\n\nrequire.undef(\'circle\');\n\ndefine(\'circle\', ["@jupyter-widgets/base", "d3"], function(widgets, d3) {\n    \n    var CircleView = widgets.DOMWidgetView.extend({\n\n        initialize: function() {\n            console.log("---- initialize, this:");\n            console.log(this);\n            this.circles = [];\n            this.radius = 1;\n            },\n\n        createDiv: function(){\n            var width = this.model.get(\'width\'); \n            var height = this.model.get(\'height\'); \n            var divstyle = $("<div id=\'d3DemoDiv\' style=\'border:1px solid red; height: " + \n                             height + "px; width: " + width + "px\'>");\n            return(divstyle);\n            },\n \n        createCanvas: function(){\n            var width = this.model.get(\'width\'); \n            var height = this.model.get(\'height\'); \n            var radius = this.model.get(\'radius\'); \n            console.log("--SIZE--", width, \'x\', height, " radius", radius);\n            var svg = d3.select("#d3DemoDiv")\n                        .append("svg")\n                        .attr("id", "svg").attr("width", width).attr("height", height);\n\n            this.svg = svg;\n            var circleView = this;\n            \n            svg.on(\'click\', function() {\n                var coords = d3.mouse(this);\n                //debugger;\n                var radius = circleView.radius;\n                console.log(\'--MOUSE--\', coords, " radius:", radius);\n                var newCircle = {x: coords[0], y: coords[1], radius: 10 * radius,\n                                 borderColor: "black", fillColor: "beige"};\n                circleView.circles.push(newCircle);\n                circleView.drawCircle(newCircle);\n                //debugger;\n                circleView.model.set("circles", JSON.stringify(circleView.circles));\n                circleView.touch();\n                });\n           }, \n\n        drawCircle: function(obj){\n           this.svg.append("circle")\n              .style("stroke", "gray")\n              .style("fill", "white")\n              .attr("r", obj.radius)\n              .attr("cx", obj.x)\n              .attr("cy", obj.y)\n              .on("mouseover", function(){d3.select(this).style("fill", "aliceblue");})\n              .on("mouseout",  function(){d3.select(this).style("fill", "white");});\n            },\n\n        render: function() { \n            this.$el.append(this.createDiv());\n            this.listenTo(this.model, \'change:circles\', this._circles_changed, this);\n            this.listenTo(this.model, \'change:radius\', this._radius_changed, this);\n            var circleView = this;\n            function myFunc(){\n               circleView.createCanvas()\n               };\n            setTimeout(myFunc, 500);\n            },\n\n        _circles_changed: function() {\n           var circles = this.model.get("circles");\n           var newCircle = circles[circles.length-1];\n           console.log(\'--DRAW--\', this.circles);\n           this.circles.push(newCircle);\n           console.log(\'--LENGTH--\', circles.length, " == ", circles.length);\n           this.drawCircle(newCircle);\n           },\n\n        _radius_changed: function() {\n           console.log(\'--RADIUS--\', this.radius, this.model.get(\'radius\'));\n           this.radius = this.model.get(\'radius\');\n           }\n    });\n    return {\n        CircleView : CircleView\n    };\n});\n')


# In[17]:


cw = CircleWidget(width=500, height=100)
scale = ipywidgets.IntSlider(1, 0, 10)
box = widgets.VBox([scale, cw])
mylink = ipywidgets.jslink((cw, 'radius'), (scale, 'value'))
box


# In[18]:


cw.drawCircle(x=30, y=30)


# In[19]:


scale.value = 2


# In[20]:


cw.drawCircle(x=60, y=30)


# In[21]: